#!/usr/bin/env python3

import sys
import os
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..', '..', 'scripts'))

from obt.cpp_db_files import main

if __name__ == '__main__':
    main()